//
//  SnowFallView.m
//  SnowGlobe
//
//  Created by SJSU Student on 11/23/10.
//  Copyright 2010 SJSU. All rights reserved.
//

#import "SnowFallView.h"

// Define the number of snowflakes to draw on the view.  NOTE: more than 40
// works well in simulator, but has some performance issues on an iPhone 4.
#define NUM_SNOWFLAKES 30

@implementation SnowFallView

@synthesize xStep, yStep;	// user adjustable flake motion per frame
NSMutableArray *flakes;		// this array holds all the snowflake objects

// images to use for the snowflakes
UIImage *flakeImage1, *flakeImage2, *flakeImage3, *flakeImage4;

// This view should be loaded from the XIB file, so need to overrride 
// initWithCoder to perform additional setup, namely creating the snowflakes.

- (id)initWithCoder:(NSCoder *)decoder {

	// initial flake fall rates
	xStep = 1.0;
	yStep = 2.0;
    if ((self = [super initWithCoder:decoder])) {
        // Initialization code
		flakes = [[NSMutableArray alloc] init];
		int i;
		for(i=0;i<NUM_SNOWFLAKES;i++){	// add flakes to view
			[flakes addObject: [[SnowFlake alloc] init]];
		}
		// load the flake images for use
		flakeImage1 = [UIImage imageNamed:@"snowflake1.png"];
		flakeImage2 = [UIImage imageNamed:@"snowflake2.png"];
		flakeImage3 = [UIImage imageNamed:@"snowflake3.png"];
		flakeImage4 = [UIImage imageNamed:@"snowflake4.png"];
    }
    return self;
}


// Override the drawing method to draw snow falling
- (void)drawRect:(CGRect)rect {
    // Drawing code
	
	for (SnowFlake *flake in flakes) {
		// create a CoreGraphics point representing the flake's location
		CGPoint drawPoint = CGPointMake(flake.xPos, flake.yPos);
		// pick the appropraite image to draw for the given flake
		switch (flake.flakeType) {
			case 0:
				[flakeImage1 drawAtPoint: drawPoint];
				break;
			case 1:
				[flakeImage2 drawAtPoint: drawPoint];
				break;
			case 2:
				[flakeImage3 drawAtPoint: drawPoint];
				break;
			case 3:
				[flakeImage4 drawAtPoint: drawPoint];
				break;
			default:
				NSLog(@"Snowflake type not recognized");
				break;
		}

		// update the flake's position
		flake.xPos+=xStep;
		// offset the universal yStep by individual flake's fall rate for 
		// more interesting/realistic visual
		flake.yPos+=flake.fallrate+yStep;
		
		// if it has fallen off the screen, regenerate near top
		if(flake.yPos>460 || flake.yPos < -100 || flake.xPos > 500 || flake.xPos < -100) 
		{
			flake.xPos = arc4random() % ((unsigned)RAND_MAX + 1) % 480;
			flake.yPos = arc4random() % ((unsigned)RAND_MAX + 1) % 60 - 20;
			// possibly slightly faster, but not as random...
			//flake.xPos = rand() % 480;
			//flake.yPos = rand() % 40 - 20;
		}
	}
}

// This resets state after a snowfall event ends, makes ready for next snowfall
- (void)reset {
	self.hidden = YES;
	self.alpha = 1.0;
	self.yStep = 2.0;
	self.xStep = 1.0;
}

- (void)dealloc {
	[flakes release];
    [super dealloc];
}

@end
